import { Extension } from 'resource:///org/gnome/shell/extensions/extension.js';
import St from 'gi://St';
import GObject from 'gi://GObject';
import Gio from 'gi://Gio';
import * as Main from 'resource:///org/gnome/shell/ui/main.js';
import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';
class BluetoothPanelButton extends PanelMenu.Button {
    static {
        GObject.registerClass(this);
    }
    //Retrieve all paired Bluetooth devices
    async getDevices() {
        /* Gio.Subprocess */
        Gio._promisify(Gio.Subprocess.prototype, 'communicate_utf8_async');
        let deviceInfos = [];
        const proc = Gio.Subprocess.new(['bluetoothctl', 'devices'], Gio.SubprocessFlags.STDOUT_PIPE | Gio.SubprocessFlags.STDERR_PIPE);
        //@ts-ignore
        const [stdout, stderr] = await proc.communicate_utf8_async(null, null);
        if (!proc.get_successful()) {
            Main.notify('Make sure you have bluetoothctl installed to run this extension.');
            deviceInfos.push({ id: "", name: "Devices could not" });
            deviceInfos.push({ id: "", name: "be detected." });
            deviceInfos.push({ id: "", name: "Make sure" });
            deviceInfos.push({ id: "", name: "bluetoothctl" });
            deviceInfos.push({ id: "", name: "is installed." });
            throw new Error(stderr);
        }
        if (stdout == '' || stdout == '\n') {
            deviceInfos.push({ id: "", name: "No paired devices." });
            deviceInfos.push({ id: "", name: "Make sure" });
            deviceInfos.push({ id: "", name: "bluetoothctl" });
            deviceInfos.push({ id: "", name: "is installed." });
            return deviceInfos;
        }
        stdout.split('\n').forEach((deviceInfo) => {
            if (deviceInfo == '')
                return; //Because the last actual line ends with '\n', so there's a blank last line that we don't want to include.
            deviceInfos.push({
                //deviceInfo is formatted like 'Device 00:00:00:00:00:00 Name of Device'
                id: deviceInfo.substring(7, 25).trim(),
                name: deviceInfo.substring(25)
            });
        });
        return deviceInfos;
    }
    constructor() {
        super(0.5, 'Bluetooth Devices');
        const icon = new St.Icon({
            icon_name: 'bluetooth-symbolic',
            style_class: 'system-status-icon'
        });
        this.add_child(icon);
        this.getDevices().then((devices) => {
            devices.forEach((device) => {
                let pmItem;
                // Check if a phone symbol should be added next to device. 
                //***
                const phoneSymbolProcess = Gio.Subprocess.new(
                // The program and command options are passed as a list of arguments
                ['bluetoothctl', 'info', device.id], 
                // The flags control what I/O pipes are opened and how they are directed
                Gio.SubprocessFlags.STDOUT_PIPE | Gio.SubprocessFlags.STDERR_PIPE);
                phoneSymbolProcess.communicate_utf8_async(null, null, (phoneSymbolProcess, res) => {
                    try {
                        if (!phoneSymbolProcess)
                            throw new Error("phoneSymbolProcess is null");
                        let [, stdout, stderr] = phoneSymbolProcess.communicate_utf8_finish(res);
                        // Failure
                        if (!phoneSymbolProcess.get_successful())
                            throw new Error(stderr);
                        // Success
                        pmItem = new PopupMenu.PopupImageMenuItem(device.name, stdout.indexOf("Icon: phone") >= 0 ? 'phone-symbolic' : '');
                        //@ts-ignore
                        this.menu.addMenuItem(pmItem);
                        addOnClickListener(pmItem);
                    }
                    catch (e) {
                        logError(e);
                    }
                });
                //***
                // Open the file dialog when a bluetooth device is clicked.
                //***
                function addOnClickListener(pmItem) {
                    pmItem.connect('activate', () => {
                        const proc = Gio.Subprocess.new(
                        // The program and command options are passed as a list of arguments
                        ['bluetooth-sendto', `--device=${device.id}`, `--name=${device.name}`], 
                        // The flags control what I/O pipes are opened and how they are directed
                        Gio.SubprocessFlags.STDOUT_PIPE | Gio.SubprocessFlags.STDERR_PIPE);
                        proc.communicate_utf8_async(null, null, (proc, res) => {
                            try {
                                if (!proc)
                                    throw new Error('proc is null.');
                                let [, , stderr] = proc.communicate_utf8_finish(res);
                                // Failure
                                if (!proc.get_successful())
                                    throw new Error(stderr);
                            }
                            catch (e) {
                                logError(e);
                            }
                        });
                    });
                }
                //***
            });
        });
    }
}
export default class BluetoothFileSenderExtension extends Extension {
    /**
     * This class is constructed once when your extension is loaded, not
     * enabled. This is a good time to setup translations or anything else you
     * only do once.
     *
     * You MUST NOT make any changes to GNOME Shell, connect any signals or add
     * any event sources here.
     *
     * @param {ExtensionMeta} metadata - An extension meta object
     */
    constructor(metadata) {
        super(metadata);
        console.debug(`constructing ${this.metadata.name}`);
    }
    /**
     * This function is called when your extension is enabled, which could be
     * done in GNOME Extensions, when you log in or when the screen is unlocked.
     *
     * This is when you should setup any UI for your extension, change existing
     * widgets, connect signals or modify GNOME Shell's behavior.
     */
    enable() {
        console.debug(`enabling ${this.metadata.name}`);
        //@ts-ignore
        this._indicator = new BluetoothPanelButton();
        //@ts-ignore
        Main.panel.addToStatusArea('myPopup', this._indicator, 1);
    }
    /**
     * This function is called when your extension is uninstalled, disabled in
     * GNOME Extensions or when the screen locks.
     *
     * Anything you created, modified or setup in enable() MUST be undone here.
     * Not doing so is the most common reason extensions are rejected in review!
     */
    disable() {
        console.debug(`disabling ${this.metadata.name}`);
        //@ts-ignore
        if (this._indicator) {
            //@ts-ignore
            this._indicator.destroy();
            //@ts-ignore
            delete this._indicator;
        }
    }
}
